; SNES SPC-700 CPU DIV Instruction Test
;
; Tests the DIV instruction, including the quotient, remainder, and overflow
; flag. Doesn't test the negative, zero, or half carry flags. Assumes that the
; result of DIV YA,X depends only on the values of Y, A, and X.
;
; Beeps every three seconds for about 13 minutes, then plays continuous tone.
; If test fails at some point, beeps twice then stops.
;
; Iterates over denominators from $00 to $ff. For each denominator, iterates
; over numerators from $0000 to $ffff. The result follows a regular pattern,
; where d is denominator and 'x n' means that value is repeated 'n' times.
; The pattern for the quotient is:
;
; $00 x d, $01 x d ... $fe x d, $ff x d
; $00 x d, $01 x d ... $fe x d, $ff x d
; $ff x (256 - d), $fe x (256 - d) ... until numerator reaches $ffff
;
; The remainder is:
;
; [$00, $01 ... d - 1] x 512
; [d, d + 1 ... $ff] repeating until numerator reaches $ffff

; Tested on SNES and in Game_Music_Emu's Spc_Emu player.
; Generates SPC file. Written for the wla assembler.
; hotpop.com@blargg (swap to e-mail)

.include "spc_runtime.asm"
      
      .define numer     2
      .define numer_h   3
      .define denom     4
      .define quot      5
      .define rem       6
      
      mov   denom,#0
denom_loop:
      mov   x,#$ff            ; reload stack pointer
      mov   sp,x
      
      mov   a,#1              ; regular beep for each iteration
      call  !make_beeps
      
      mov   numer,#$00        ; numer = 0
      mov   numer_h,#$00
      mov   x,denom           ; load denominator
      beq   zero_denom
      
                              ; non-zero denominator
                              
      mov   a,#$70            ; bvs (overflow should be clear during loop)
      mov   !normal_branch,a
      call  !check_normal
      
      mov   a,#$50            ; bvs (overflow should be set during loop)
      mov   !normal_branch,a
      call  !check_normal
      
zero_denom:
      call  !check_overflow
      
test_passed:
      inc   denom
      bne   denom_loop
      
      call  !start_tone       ; continuous tone = success
      jmp   !exit
      
check_normal:
      mov   quot,#$00
normal_loop:
      mov   rem,#0
normal_loop2:
      movw  ya,numer          ; check div
      div   ya,x
normal_branch:
      bvc   error             ; overflow
      cmp   a,quot            ; quotient
      bne   error
      cmp   y,rem             ; remainder
      bne   error
      incw  numer             ; stop when numer passes 0xffff
      beq   test_passed
      
      inc   rem               ; next value
      cmp   x,rem
      bne   normal_loop2
      inc   quot
      bne   normal_loop
      ret

check_overflow:
      mov   quot,#$ff
overflow_loop:
      mov   rem,x
overflow_loop2:
      movw  ya,numer          ; check div
      div   ya,x
      bvc   error             ; overflow
      cmp   a,quot            ; quotient
      bne   error
      cmp   y,rem             ; remainder
      bne   error
      incw  numer             ; stop when numer passes 0xffff
      beq   test_passed
      
      inc   rem               ; next value
      bne   overflow_loop2
      dec   quot
      jmp   !overflow_loop
      
error:
      mov   a,#2
      call  !make_beeps
      jmp   !exit
      
.include "util.asm"
